@@ -1,13 +1,10 @@
 package CGI::Lazy::DB::RecordSet;
 
 use strict;
-use warnings;
 
 use Tie::IxHash;
 use Data::Dumper;
 
-no warnings qw(uninitialized redefine);
-
 #------------------------------------------------------------------
 sub basewhere {
 	my $self = shift;
@@ -57,6 +54,8 @@ sub createSelect {
 		unless ($self->displayOnly($field)) {
 			if ($self->readfunc($field)) {
 				push @fieldlist, $self->readfunc($field);
+			} elsif ($self->passwd($field)) {
+				next;
 			} else {
 				push @fieldlist, $field;
 			}
@@ -104,6 +103,7 @@ sub delete {
 	foreach my $ID (keys %$data) {
 		my $query = "delete from $table where $primarykey = ?";
 	       
+		${$self->primarykeyhandle} = $ID;
 #		$self->q->util->debug->edump($query.", $ID");
 		$self->db->do($query, $ID);
 
@@ -206,7 +206,7 @@ sub insert {
 						${$vars->{$field}->{handle}} = $defaults->{$field}->{value};
 					}
 				} else { #values pulled from queries and such
-					my $result = $self->db->getarray($defaults->{$field}->{sql});
+					my $result = $self->db->getarray(@{$defaults->{$field}->{sql}});
 
 					if (defined $result->[1] || defined $result->[0]->[1]) { #we got more than a single value, better warn
 						$self->q->errorHandler->dbReturnedMoreThanSingleValue;
@@ -232,21 +232,36 @@ sub insert {
 				unless ($self->displayOnly($field) || $self->readOnly($field)) {
 					push @fieldlist, $field;
 
-					if ($vars->{$field}->{handle}) {
-						${$vars->{$field}->{handle}} = $data->{$row}->{$field};
-					}
+					my $value;
+
 
 					if ($self->inputMask($field)) {
-						push @bindvalues, sprintf $self->inputMask($field), $data->{$row}->{$field};
+						$value = sprintf $self->inputMask($field), $data->{$row}->{$field};
+					} elsif ($self->passwd($field)){
+						if ($self->q->authn) {
+							$value = $self->q->authn->passwdhash($data->{$row}->{$self->passwd($field)->{userField}}, $data->{$row}->{$field});
+						}
 					} else {
-						push @bindvalues, $data->{$row}->{$field};
+						$value = $data->{$row}->{$field};
 					}
 
+					if ($vars->{$field}->{handle}) {
+						${$vars->{$field}->{handle}} = $value;
+					}
+
+					if ($field eq $self->primarykey) {
+						${$self->primarykeyhandle} = $value;
+
+					}
+
+					push @bindvalues, $value;
+
 					if ($self->writefunc($field) ) {
 						push @binds,  $self->fieldlist->{$field}->{writefunc};
 					} else {
 						push @binds, "?";
 					}
+
 				}
 			}
 		}
@@ -263,14 +278,10 @@ sub insert {
 			${$self->primarykeyhandle} = $self->db->get($query);
 		}
 
+
 		if ($additional) { #addional queries run on insert
 			foreach my $field (keys %$additional) {
-				my $result;
-				if (ref $additional->{$field}->{sql}) {
-					$result = $self->db->getarray(@{$additional->{$field}->{sql}});
-				} else {
-					$result = $self->db->getarray($additional->{$field}->{sql});
-				}
+				my $result = $self->db->getarray(@{$additional->{$field}->{sql}});
 
 				if (defined $result->[1] || defined $result->[0]->[1]) { #we got more than a single value, better warn
 					$self->q->errorHandler->dbReturnedMoreThanSingleValue;
@@ -337,7 +348,7 @@ sub new {
 		_updateadditional	=> $args->{updateadditional},
 		_where			=> '',
 		_mysqlAuto		=> $args->{mysqlAuto},
-		_primarykeyhandle	=> \$var,
+		_primarykeyhandle		=> \$var,
 		_checkboxes		=> [],
 
 	};
@@ -445,6 +456,22 @@ sub mysqlAuto {
 
 }
 
+#------------------------------------------------------------------------------
+sub passwd {
+	my $self = shift;
+	my $field = shift;
+	
+	if (exists $self->fieldlist->{$field}) {
+		if ($self->fieldlist->{$field}->{passwd}) {
+			return $self->fieldlist->{$field}->{passwd};
+		} else {
+			return;
+		}
+	} else {
+		return;
+	}
+}
+
 #------------------------------------------------------------------------------
 sub primarykey {
 	my $self = shift;
@@ -518,7 +545,7 @@ sub select {
 	eval {
 		$sth = $self->db->dbh->prepare($query);
 		$sth->execute(@bindvars);
-#		$self->q->util->debug->edump($query, @bindvars);
+		#$self->q->util->debug->edump($query, @bindvars);
 	};
 
 	if ($@) {
@@ -532,6 +559,7 @@ sub select {
 			tie (%$record, 'Tie::IxHash');
 
 			for (0..$#fieldlist) {
+				next if	$self->passwd($fieldlist[$_]);
 				$record->{$fieldlist[$_]} = $record[$_];
 			}
 
@@ -541,6 +569,7 @@ sub select {
 
 	$self->{_data} = \@data; 
 
+	#$self->q->util->debug->edump(\@data);
 	return $self->{_data};
 }
 
@@ -588,7 +617,7 @@ sub update {
 						${$vars->{$field}->{handle}} = $defaults->{$field}->{value};
 					}
 				} else { #values pulled from queries and such
-					my $result = $self->db->getarray($defaults->{$field}->{sql});
+					my $result = $self->db->getarray(@{$defaults->{$field}->{sql}});
 
 					if (defined $result->[1] || defined $result->[0]->[1]) { #we got more than a single value, better warn
 						$self->q->errorHandler->dbReturnedMoreThanSingleValue;
@@ -612,16 +641,35 @@ sub update {
 				unless ($self->displayOnly($field) || $self->readOnly($field)) {
 					if ($vars->{$field}->{handle}) {
 						${$vars->{$field}->{handle}} = $data->{$ID}->{$field};
+
+					}
+
+					if ($field eq $self->primarykey) {
+						${$self->primarykeyhandle} = $data->{$ID}->{$field};
+
 					}
 
 					if ($self->inputMask($field)) {
 						push @binds, sprintf $self->inputMask($field), $data->{$ID}->{$field};
+					} elsif ($self->passwd($field)){
+						if ($data->{$ID}->{$field}) {
+							if ($self->q->authn) {
+								push @binds, $self->q->authn->passwdhash($data->{$ID}->{$self->passwd($field)->{userField}}, $data->{$ID}->{$field});
+							}
+						}
 					} else {
 						push @binds, $data->{$ID}->{$field};
 					}
+
 					if ($self->writefunc($field) ) {
 						push @updates,  "$field = ".$self->fieldlist->{$field}->{writefunc};
 
+					} elsif ($self->passwd($field)) {
+						if ($self->q->authn) {
+							if ($data->{$ID}->{$field}) {
+								push @updates,  "$field = ?";
+							}
+						}
 					} else {
 						push @updates,  "$field = ?";
 					}
@@ -651,14 +699,11 @@ sub update {
 	       
 		$self->db->do($query, @binds, $ID);
 
+		${$self->primarykeyhandle} = $ID;
+
 		if ($additional) { #addional queries run on insert
 			foreach my $field (keys %$additional) {
-				my $result;
-				if (ref $additional->{$field}->{sql}) {
-					$result = $self->db->getarray(@{$additional->{$field}->{sql}});
-				} else {
-					$result = $self->db->getarray($additional->{$field}->{sql});
-				}
+				my $result = $self->db->getarray(@{$additional->{$field}->{sql}});
 
 				if (defined $result->[1] || defined $result->[0]->[1]) { #we got more than a single value, better warn
 					$self->q->errorHandler->dbReturnedMoreThanSingleValue;
@@ -1152,6 +1197,29 @@ Options:
 
 			},
 
+			{
+
+				name		=> 'username',
+
+				label		=> 'Username',
+
+			},
+
+			{	
+
+				name		=> 'passwd',
+				
+				label		=> 'Password',
+
+				passwd		=> {
+
+							userField	=> 'username',
+
+				},
+
+			},
+				
+
 		],
 
 		insertdefaults	=> {
@@ -1164,12 +1232,12 @@ Options:
 
 			field2 => {
 
-				sql	=> 'select foo from bar',
+				sql	=> ['select foo from bar where ?', $bind1],
 			},
 
 			field3	=> {
 
-				sql	=> 'select foo.nextvar from dual',
+				sql	=> ['select foo.nextvar from dual'],
 				handle	=> $ref,
 			}
 
@@ -1183,12 +1251,12 @@ Options:
 
 			field2 => {
 
-				sql	=> 'select foo from bar',
+				sql	=> ['select foo from bar where ?', $bind1],
 			},
 
 			field3	=> {
 
-				sql	=> 'select foo.nextvar from dual',
+				sql	=> ['select foo.nextvar from dual'],
 				handle	=> $ref,
 			}
 
@@ -1252,7 +1320,7 @@ array ref. list of fields with their attributes
 		
 		value	=> value to insert
 
-		sql	=> sql to generate value to insert
+		sql	=> array ref.  first value is sql to generate value to insert, rest is binds
 
 		handle	=> reference whose referrent will contain whatever value is set into db.  Useful for later use in cgi.
 
@@ -1260,7 +1328,7 @@ array ref. list of fields with their attributes
 		
 		value	=> value to insert
 
-		sql	=> sql to generate value to insert
+		sql	=> array ref. first value is sql to generate value to insert, rest is binds
 
 		handle	=> reference whose referrent will contain whatever value is set into db.  Useful for later use in cgi.
 
@@ -1276,6 +1344,10 @@ array ref. list of fields with their attributes
 
 		notNull	=> 1   Set this for selects if you don't want the first item of the select to be blank
 
+	passwd		=> This field is password field used with the authn plugin
+
+		userField	=> name of field in recordset that will contain the username
+
 
 =head3 updatedefaults 
 
@@ -1324,6 +1396,18 @@ Returns arrayref or array of fields flagged to show up on multiple records page
 
 Returns arrayref or array of labels for fields chosen to appear on multiple record pages.
 
+=head2 mysqlAuto ()
+
+Returns true if recordset was created with mysqlAuto => 1 .
+
+=head2 passwd ( field ) 
+
+Returns passwd hashref from fieldlist config.  
+
+=head3 fieldname
+
+name of the field in question
+
 =head2 primarykey ( fieldname )
 
 returns or sets the primary key for the object
@@ -1334,11 +1418,7 @@ The name of the field in the database
 
 =head2 primarykeyhandle ()
 
-Returns scalar ref to primary key if set via default or mysqlAuto.
-
-=head2 mysqlAuto ()
-
-Returns true if recordset was created with mysqlAuto => 1 .
+Returns scalar ref to primary key of record being processed.  Used by Composite widget to get primary key of parent.  Only really useful for datasets in single mode.  Use with multiple row datasets at your own risk.
 
 =head2 q ()
 
